#################################################################################
# Created       : Martin Foster
# Modified      : 08/28/2001
#################################################################################
#
# Table - Modules part of Webchat designed to simplify table creation  
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Table;					# Package name

use strict;							# Strict variable enforcement

#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;					# Self reference
	my $cgi   = shift;					# CGI handle

	my $self = {CGI => $cgi};
	bless $self, $class;					# Bless $self into designated class
	return $self;
}



#####################
# Make Table
#
# Table contruction sub-routine, designed to hopefully cut down on code for certain
# apects of scripts needed it.

sub Make
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $choice   = shift;					# Decision to make

	my $cgi      = $self->{CGI};				# CGI handle
	my $width;						# Table width

	#####################
	# Program area

	# Top of table
	if ($choice == 0)
	{
		# Verify value
		$width = shift;
		$width = (defined($width)) ? $width : 500;

		# Table top
		print "<CENTER><TABLE WIDTH=$width COLS=3>\n"; 
	}

	# blank row of table
	elsif ($choice == 1)
	{ print "\t<TR>\n\t\t<TD HEIGHT=15 COLPSAN=3>&nbsp;</TD>\n\t</TR>\n"; }

	# Filled row of table (Invalid)
	elsif ($choice == 2) 
	{
		print "\t<TR>\n";				# Beginning of row

		print "\t\t<TD>$_[0]</TD>\n";
		print "\t\t<TD WIDTH=10><FONT COLOR=\"\#FF0000\"><B>*</B></FONT></TD>\n";
		print "\t\t<TD>$_[1]</TD>\n";

		print "\t</TR>\n";				# End of row
	}

	# Filled row of table (Done)
	elsif ($choice == 3)
	{
		print "\t<TR>\n";				# Beginning of row

		print "\t\t<TD>$_[0]</TD>\n";
		print "\t\t<TD WIDTH=10>&nbsp;</TD>\n";
		print "\t\t<TD>$_[1]</TD>\n";

		print "\t</TR>\n";				# End of row
	}

	# Filled row of table (Changed)
	elsif ($choice == 4)
	{
		print "\t<TR>\n";				# Beginning of row

		print "\t\t<TD>$_[0]</TD>\n";
		print "\t\t<TD WIDTH=10><FONT COLOR=\"\#009900\"><B>*</B></FONT></TD>\n";
		print "\t\t<TD>$_[1]</TD>\n";

		print "\t</TR>\n";				# End of row
	}


	# Tertiary Rows
	elsif ($choice == 5)
	{
		print "\t<TR>\n";				# Beginning of row

		print "\t\t<TD>$_[0]</TD>\n";
		print "\t\t<TD>$_[1]</TD>\n";
		print "\t\t<TD>$_[2]</TD>\n";

		print "\t</TR>\n";				# End of row
	}

	# Single row
	elsif ($choice == 6)
	{
		print "\t<TR>\n";				# Beginning of row
		print "\t\t<TD COLSPAN=3>$_[0]</TD>\n";
		print "\t</TR>\n";				# End of row
	}

        # bottom of table
        elsif ($choice == 7)
        { print "</TABLE></CENTER>\n"; }
}

# Alias for top
sub MakeTop
{ Make(shift, 0, shift); }

# Alias for bottom
sub MakeBottom
{ Make(shift, 7); }

# Alias for Blank rows
sub MakeBlank
{ Make(shift, 1); }

# Alias for Invalid entries
sub MakeInvalid
{ Make(shift, 2, @_); }

# Alias for Valid entries
sub MakeValid
{ Make(shift, 3, @_); }

# Alias for Changed entries
sub MakeChanged
{ Make(shift, 4, @_); }

# Alis for all three columns
sub MakeTertiary
{ Make(shift, 5, @_); }

# Alias for single
sub MakeSingle
{ Make(shift, 6, @_); }
						
1;								# Return true
