#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 05/16/2002
#################################################################################
#
# Gummy and Font - Script part of Webchat designed to manage gummies 
# rights.  Contains comprehensive search parameter.
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################
use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Audit trail
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my $scriptname;							# Script name
my $table;							# Table handle

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi); 


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationAdmin())
	{
		# Define table handler
		$table = new Ethereal::Table($cgi);

                # Pull scriptname
                $scriptname = $cgi->url(-relative=>1);


		# Gummy handler
		if ($scriptname =~ /^gummy/i)
		{
			# Title
			print $cgi->h1('Gummy Administration'), "\n";

			# Class selection
			unless (defined($param->{'CLASS'}))
			{
				# Selection of class
				GummySelect($database, $cgi, $param, $table);
			}

			else
			{
				# Handle addition and removal
				GummyHandler($database, $cgi, $param, $table);
			}
		}

		# Font handler
		elsif ($scriptname =~ /^font/i)
		{
			# Title
			print $cgi->h1('Font Administration'), "\n";

			# Font handling subsystem
			FontHandler($database, $cgi, $param, $table);
		}
	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# FontHandler
#
# Will simply display the font colour, face and size as to allow the administrator
# to add and remove them as need be.   Similiar in function to the gummy handler.

sub FontHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter reference
	my $table    = shift;					# Table parameter

	my $audit;						# Auditing handle
	my $data     = $database->{'HANDLE'};			# Database handle

	my $res;						# Results			
	my $statement;						# Database statement

	my $name;						# Font name

	my $ctype;						# Current working type
	my $type;						# Font type


	#####################
	# Program area

	# Audit handle
	$audit = new Ethereal::Audit($database, $cgi, $param);


	# Additions
	if ((defined($param->{'NAME'}))
	 && (defined($param->{'CODE'}))
	 && (length($param->{'NAME'}) > 0))
	{
		# Force lower case
		$param->{'TYPE'} = lc($param->{'TYPE'});

		# Insert information
		$data->do("INSERT INTO Font
			(FontName,
			 FontType,
			 FontCode)
			VALUES (?,?,?)", {},

		 $param->{'NAME'},
		 $param->{'TYPE'},
		 $param->{'CODE'});


		# Audit
		$audit->FontRem($param->{'TYPE'},$param->{'NAME'});

		# Cleanup
		$param->Cleanup('CODE','NAME','TYPE');
	}



	# Query management
	# Prepare and execute
	$statement = $data->prepare("SELECT 
		 FontName,
		 FontType,
		 FontCode
		FROM  Font
		ORDER BY FontType, FontName");
	$statement->execute();


	# Form handle
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, 'USER', $param->{'USER'}, 'CRYPT',  $param->{'CRYPT'});

	# HTML display
	# Table Top
	print "<CENTER>\n<TABLE WIDTH=500 CELLPADDING=5>\n";

	# Cycle through gummies
	while ($res = $statement->fetchrow_hashref())
	{
		# Title
		if ((!defined($type))
		 or ($type ne $res->{'FontType'}))
		{
			# Uppercase
			$type = uc($res->{'FontType'});


			# Display row
			print " <TR>\n",
			      "  <TD COLSPAN=3><BR>\n<BR>\n<B><FONT SIZE=+2>$type</FONT></B><BR></TD>\n",
			      " </TR>\n";


			# Alternate for next pass
			$type = $res->{'FontType'};
		}

		# Name handling
		$name = $res->{FontName};
		$name =~ s/\s//gs;

		# Handle removal
		if (defined($param->{$name}))
		{
			# Remove gummy
			$data->do("DELETE FROM Font
				WHERE FontName=?
				  AND FontType=?", {},

			 $res->{'FontName'},
			 $res->{'FontType'});


			# Audit action
			$audit->FontRem($res->{'FontType'},$res->{'FontName'});

			# Cleanup
			$param->Cleanup($name);
		}

		# Display if not removed
		else
		{
			# Display row
			if ($type eq 'color')
			{
				# Display colour
				print " <TR>\n",
				      "  <TD><B>$res->{FontName}</B></TD>\n",
				      "  <TD BGCOLOR=\"$res->{FontCode}\" WIDTH=15>&nbsp;</TD>\n",
				      "  <TD><CENTER>", $cgi->checkbox(-name=>$name, -label=>''), "</CENTER></TD>\n",
				      " </TR>\n";
			}
			else
			{
				# Current working type
				$ctype = uc($type);

				# Simply show a blank space
				print " <TR>\n",
				      "  <TD><FONT $ctype=\"$res->{FontCode}\">$res->{FontName}</FONT></TD>\n",
				      "  <TD>&nbsp;</TD>\n",
				      "  <TD><CENTER>", $cgi->checkbox(-name=>$name, -label=>''), "</CENTER></TD>\n",
				      " </TR>\n";
			}
		}
	}

	# Display row
	print " <TR>\n",
	      "  <TD>&nbsp;</TD>\n",
	      "  <TD>&nbsp;</TD>\n",
	      "  <TD><CENTER>", $cgi->submit('Remove'), "</CENTER></TD>\n",
	      " </TR>\n";


	# End of table
	print "</TABLE>\n</CENTER>\n";
	
	# End query
	$statement->finish();


	# Widget
	my @radiotype  = qw(Color Face Size); 
	my $radiofield = $cgi->radio_group('TYPE', \@radiotype, $radiotype[0], 'true');


	# Spacer
	print "<BR>\n<BR>\n";

	# Form handle
	# Addition section
	# Table top
	$table->MakeTop();
	$table->MakeSingle("<B><FONT SIZE=+2>ADDITION</FONT></B>");
	$table->MakeBlank();

	# Display rows
	$table->MakeValid('<B>Name</B>',  $cgi->textfield('NAME', '', 20, 20));
	$table->MakeValid('<B>Code</B>', $cgi->textfield('CODE', '', 20, 35));
	$table->MakeBlank();

	$table->MakeValid('<B>Type</B>', $radiofield);
	$table->MakeBlank();

	$table->MakeSingle($cgi->submit('Add'));
	$table->MakeBottom();
	
	
	# End form
	print $cgi->end_form(), "\n";
}

#####################
# GummyHandler
#
# Very simple sub-routine that will simply allow the user to select the class
# of gummies to search for.

sub GummyHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter reference
	my $table    = shift;					# Table parameter

	my $audit;						# Auditing handle
	my $data     = $database->{'HANDLE'};			# Database handle

	my $res;						# Results			
	my $statement;						# Database statement

	my $ident;						# Gummy indentier
	my $group    = 'None';					# Gummy grouping 
	my $name;						# Name of gummy
	my $realm    = 'system';				# Realm search


	#####################
	# Program area

	# Audit handle
	$audit = new Ethereal::Audit($database, $cgi, $param);


	# Addition
	# Make sure gummy is defined
	if ((defined($param->{'GUMMY'}))
	 && (defined($param->{'GPATH'}))
	 && (length($param->{'GUMMY'}) > 2))
	{
		# Force case
		$param->{'GROUP'} = lc($param->{'GROUP'});

		# Create entry
		$data->do("INSERT INTO Gummy
			(GummyName,
			 GummyClass,
			 GummyPath,
			 GummyIdent,
			 RealmName)
			VALUES (?,?,?,?,?)", {},

		 $param->{'GUMMY'},
		 $param->{'CLASS'},
		 $param->{'GPATH'},
		 $param->{'GROUP'},
		 $realm);


		# Audit action
		$audit->GummyAdd($realm,$param->{'GUMMY'});

		# Cleanup
		$param->Cleanup('GUMMY','GROUP','GPATH','Add');
	}



	# Query management
	# Prepare and execute
	$statement = $data->prepare("SELECT 
		 GummyName,
		 GummyPath,
		 GummyIdent
		FROM  Gummy
		WHERE GummyClass=?
		  AND RealmName=?
		ORDER BY GummyIdent, GummyName");
	$statement->execute($param->{'CLASS'}, $realm);


	# Form handle
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param});

	# HTML display
	# Table Top
	print "<CENTER>\n<TABLE WIDTH=500 CELLPADDING=5>\n";

	# Cycle through gummies
	while ($res = $statement->fetchrow_hashref())
	{
		# Padding
		$ident = (defined($res->{'GummyIdent'})) ? $res->{'GummyIdent'} : 'None';

		# Title
		if ($ident ne $group)
		{
			# Alternate
			$group = $ident;

			# Uppercase
			$ident = uc($ident);


			# Display row
			print " <TR>\n",
			      "  <TD COLSPAN=3><BR>\n<BR>\n<B><FONT SIZE=+2>$ident</FONT></B><BR></TD>\n",
			      " </TR>\n";
		}

		# Name handling
		$name = $res->{GummyName};
		$name =~ s/\s//gs;

		# Handle removal
		if (defined($param->{$name}))
		{
			# Remove gummy
			$data->do("DELETE FROM Gummy
				WHERE GummyName=?
				  AND RealmName=?", {},

			 $res->{'GummyName'},
			 $realm);


			# Audit action
			$audit->GummyRem($res->{'GummyName'},$realm);

			# Cleanup
			$param->Cleanup($name);
		}

		# Display if not removed
		else
		{
			# Display row
			print " <TR>\n",
			      "  <TD><IMG SRC=\"$res->{GummyPath}\"></TD>\n",
			      "  <TD><B>$res->{GummyName}</B></TD>\n",
			      "  <TD><CENTER>", $cgi->checkbox(-name=>$name, -label=>''), "</CENTER></TD>\n",
			      " </TR>\n";
		}
	}

	# Display row
	print " <TR>\n",
	      "  <TD>&nbsp;</TD>\n",
	      "  <TD>&nbsp;</TD>\n",
	      "  <TD><CENTER>", $cgi->submit('Remove'), "</CENTER></TD>\n",
	      " </TR>\n";


	# End of table
	print "</TABLE>\n</CENTER>\n";

	# End form
	print $cgi->end_form(), "\n";
	
	# End query
	$statement->finish();


	# Spacer
	print "<BR>\n<BR>\n";

	# Form handle
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, 'USER', $param->{'USER'}, 'CRYPT',  $param->{'CRYPT'}, 'CLASS',  $param->{'CLASS'});

	# Addition section
	# Table top
	$table->MakeTop();
	$table->MakeSingle("<B><FONT SIZE=+2>ADDITION</FONT></B>");
	$table->MakeBlank();

	# Display rows
	$table->MakeValid('<B>Name</B>',  $cgi->textfield('GUMMY', '', 20, 20));
	$table->MakeValid('<B>Group</B>', $cgi->textfield('GROUP', '', 20, 20));
	$table->MakeBlank();

	$table->MakeSingle('<B>Full Path</B>');
	$table->MakeSingle($cgi->textfield('GPATH', 'http://', 75, 200));
	$table->MakeBlank();

	$table->MakeSingle($cgi->submit('Add'));
	$table->MakeBottom();
	
	
	# End form
	print $cgi->end_form(), "\n";
}

#####################
# Gummy Selection
#
# Very simple sub-routine that will simply allow the user to select the class
# of gummies to search for.

sub GummySelect
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter reference
	my $table    = shift;					# Table handler
	my $widget;						# Displayed widget

	my @class;						# Table array


	#####################
	# Program area

	# Gummy list 
	@class  = qw(admin default canon general mod super);

	# Generate widget
	$widget = $cgi->popup_menu('CLASS', \@class, $class[0]);


	# Form handle
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param});

	# Table display
	# Top and widget
	$table->MakeTop();
	$table->MakeValid('<B>Select Class</B>', $widget);
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeSingle($cgi->submit('Select'));
	$table->MakeBottom();

	# End form
	print $cgi->end_form();
}
