#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 04/07/2002
################################################################################
#
# Donations - Allows for automation of certain parts of the donations process
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Crypt::CBC;							# Cypher block chaining
use LWP::UserAgent;						# Library for WWW access
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash

my %system;							# System hash

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Link with hash
	$database->GetHashSystem(\%system);


	# Regular display
	# Show the wares
	unless (defined($param->{'txn_id'}))
	{
		# User interface section
		DisplayDonations($database, $cgi, $param, \%system);
	}

	# Occurence of the transaction
	else
	{
		# Handle request
		HandleDonations($database, $cgi, $param, \%system);
	}


################################################################################
# Sub-Routines
################################################################################

#####################
# Display Donations
#
# This is the only interface that will be called by a regular user.  The rest
# will be handled exclusively by the PayPal interface that has been put into
# place.

sub DisplayDonations
{
	#####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $system   = shift;					# System hash

	my $cipher;						# Cipher handle

	my $custom;						# Custom value
	my $donations;						# Donations document


	#####################
	# Program area

	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Initialize encryption
		$cipher = new Crypt::CBC($system{'SetKey'}, $system{'SetKeyCipher'});

		# Genetrate custom number
		$custom = join("\f", $param->{'USER'}, time);
		$custom = $cipher->encrypt_hex($custom); 

		# Retrieve body
		$donations = $database->DocumentGetDonations();
		$donations =~ s/MCUSTOM/$custom/gs;
		$donations =~ s/MINVOICE/$custom/gs;


		# Display donations section
		print "$donations\n"; 
	}

	# Document footer
	$database->DocumentGetFooter();
}


#####################
# Handle Donations
#
# Will handle the requests from PayPal and jump the necessary hoops in order
# to satisfy them.   

sub HandleDonations
{
	#####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $system   = shift;					# System hash

	my $cipher;						# Cryptographic handle
	my $data     = $database->{'HANDLE'};			# Database handle

	my $key;						# Parameter key
	my $pair;						# Key/pair value
	my $query;						# Encoded query

	my $res;						# HTTP Request
	my $req;						# HTTP Request
	my $ua;							# User agent

	my $incr;						# Increments
	my $expire;						# Expiry

	my $time;						# Timestamp
	my $user;						# Username

	my $donation;						# Donation Email
	my $email;						# Recipient Email
	my $name;						# Name of recipient

	my @enc;						# Array of key/pairs


	#####################
	# Constants

	my $BASE = 2.50;					# Base donation
 	my $PAGE = 10.00;					# Constant donation

 	my $ROOF = 2147483647;					# Maximum size for time
 	my $YEAR = 31536000;					# Physical year


	#####################
	# Program area

	# Initialize encryption
	$cipher = new Crypt::CBC($system{'SetKey'}, $system{'SetKeyCipher'});

	# Encode parameters
	# Generate key/pair
	foreach $key (keys(%{$param}))
	{
		# Encode key correctly
		$pair = "$key=$param->{$key}";
		$pair =~ tr/ /+/; 

		# Add to array
		push(@enc, $pair);
	}


	# Create complete string
	$query = join("&", @enc, 'cmd=_notify-validate');


	# Post back to validate
	# Handles
	$ua  = new LWP::UserAgent;
	$req = new HTTP::Request('POST', $system{'SetDonationsVerify'});
	
	# Generate request
	$req->content_type('application/x-www-form-urlencoded');
	$req->content($query);

	# Fetch results
	$res = $ua->request($req);


	# Check for reply
	# Reply failed
	if ($res->is_error) 
	{
		# HTTP error
		print "content-type: text/plain\n\nERROR\n";
	}

	# Reply success
	else
	{
		# Verified transaction
		if ($res->content =~ /VERIFIED/) 
		{
			# Check the payment_status=Completed
			# Check that txn_id has not been previously processed
			# Check that receiver_email is an email address in your PayPal account

			# Completed transaction to proper location
			if (($param->{'payment_status'} =~ /completed/i)
			 && ($param->{'receiver_email'} =~ /$system->{'SetDonationsAddress'}/i)
			 && (defined($param->{'custom'})) && (length($param->{'custom'}) > 10))
			{
				# Retreive duplicate
				($res) = $database->DataGet("SELECT TransIdentifier
					FROM Transaction
					WHERE TransIdentifier=?", $param->{'txn_id'});

				# Create unique transaction
				unless (defined($res))
				{
					# Retreive values
					($user, $time) = split(/\f/, $cipher->decrypt_hex($param->{'custom'})); 
	
					# Populate database
					$data->do("INSERT INTO Transaction
						(TransIdentifier,
						 TransGross,
						 PuppeteerLogin,
						 TransTimestamp)
						VALUES (?,?,?,?)", {},
	
					 $param->{'txn_id'},
					 $param->{'payment_gross'},
					 $user,
					 $time);


					# Advanced donations
					if (($incr = int($param->{'payment_gross'} / $PAGE)) > 0)
					{
						# Retrieve values
						($expire) = $database->DataGetPuppeteerExpire($user);
						($time)   = time;


						# Current or expired
						$expire  = ($expire > $time) ? $expire : $time;

						# Increment years
						$expire += $YEAR * $incr; 

						# Value obscene
						$expire  = ($expire < $ROOF) ? $expire : $ROOF;


						# Database updates
						$data->do("UPDATE Puppeteer
							SET PuppeteerVerified=?,
							    PuppeteerHomespan=?
							WHERE PuppeteerLogin=?", {},

						 'yes',
						 $expire,
						 $user);
					}

					# Basic donation handling
					elsif ($param->{'payment_gross'} >= $BASE)
					{
						# Flip verification tag
						$data->do("UPDATE Puppeteer
							SET PuppeteerVerified=?
							WHERE PuppeteerLogin=?", {},

						 'yes',
						 $user);
					}


					# Mailed thanks
					# Data retrieval
					($donation) = $database->DocumentGetDonationsMail();
					($email)    = $database->DataGetPuppeteerEmail($user);
					($name)     = $database->DataGetPuppeteerName($user);

					
					# Define and open handle
					open(SENDMAIL, "|$system->{'LocSendmail'} -oi -t")
						or die("Cannot fork sendmail: $!");

					# Build Email
					print SENDMAIL "From: $system->{SetInfoContactName} <$system->{SetInfoContactAddress}>\n";
					print SENDMAIL "To: $name <$email>\n";
					print SENDMAIL "Subject: Your contribution is appreciated\n\n";
					print SENDMAIL "$donation\n";

					# Close handle
					close(SENDMAIL);
				}
			}
		}

		# Display message
		print "content-type: text/plain\n\nOK\n";
	}
}
